package belka.parser;

/**
 * The class represents set of functions aimed to resolve format of file by
 * extension.
 *
 * @author Alexej Abyzov
 */ 
public class FormatResolver
{
    /**
     * Constant to define PDB-file format
     */ 
    public final static int FORMAT_UNKNOWN = -1;
    public final static int FORMAT_PDB     =  0;
    public final static int FORMAT_PROJECT =  1;
    public final static int FORMAT_GROUP   =  2;

    /**
     * Standard extensions for file in PDB format
     */ 
    public final static String FORMAT_PDB_EXTENSIONS[] = { ".pdb",
							   ".ent",
							   ".noc",
							   ".pdb.gz",
							   ".ent.gz",
							   ".noc.gz"};

    /**
     * Standard extensions for project file format
     */ 
    public final static String FORMAT_PROJECT_EXTENSIONS[] = { ".bel"};

    /**
     * Standard extensions for annotation/group file format
     */ 
    public final static String FORMAT_GROUP_EXTENSIONS[] = { ".gr"};

    /**
     * The function resolves file format by its extension.
     *
     * @param fileName name of the file
     *
     * @return format index, or negative if format can no be resolved.
     */
    public static int resolveFileFormatByExtension(String fileName)
    {
	if (fileName == null) return FORMAT_UNKNOWN;

	for (int i = 0;i < FORMAT_PDB_EXTENSIONS.length;i++)
	    if (fileName.endsWith(FORMAT_PDB_EXTENSIONS[i]))
		return FORMAT_PDB;

	for (int i = 0;i < FORMAT_PROJECT_EXTENSIONS.length;i++)
	    if (fileName.endsWith(FORMAT_PROJECT_EXTENSIONS[i]))
		return FORMAT_PROJECT;

	for (int i = 0;i < FORMAT_GROUP_EXTENSIONS.length;i++)
	    if (fileName.endsWith(FORMAT_GROUP_EXTENSIONS[i]))
		return FORMAT_GROUP;

	return FORMAT_UNKNOWN;
    }
}